@@ -1,19 +1,33 @@
 # -*- coding: utf-8 -*-
 """The main Process module"""
 import abc
+import asyncio
 import contextlib
+import copy
 import enum
 import functools
-import copy
 import logging
-import time
+import re
 import sys
-import uuid
-import asyncio
+import time
 from types import TracebackType
 from typing import (
-    Any, Awaitable, Callable, cast, Dict, Generator, Hashable, List, Optional, Sequence, Tuple, Type, Union
+    Any,
+    Awaitable,
+    Callable,
+    Dict,
+    Generator,
+    Hashable,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+    cast,
 )
+import uuid
+import warnings
 
 try:
     from aiocontextvars import ContextVar
@@ -21,23 +35,16 @@ except ModuleNotFoundError:
     from contextvars import ContextVar
 
 from aio_pika.exceptions import ConnectionClosed
-import yaml
 import kiwipy
+import yaml
 
+from . import events, exceptions, futures, persistence, ports, process_comms, process_states, utils
+from .base import state_machine
+from .base.state_machine import StateEntryFailed, StateMachine, TransitionFailed, event
+from .base.utils import call_with_super_check, super_check
 from .process_listener import ProcessListener
 from .process_spec import ProcessSpec
-from .utils import protected, PID_TYPE, SAVED_STATE_TYPE
-from . import exceptions
-from . import futures
-from .base import state_machine
-from .base.utils import super_check, call_with_super_check
-from .base.state_machine import event, StateEntryFailed, StateMachine, TransitionFailed
-from . import events
-from . import persistence
-from . import ports
-from . import process_comms
-from . import process_states
-from . import utils
+from .utils import PID_TYPE, SAVED_STATE_TYPE, protected
 
 # pylint: disable=too-many-lines
 
@@ -178,10 +185,10 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
                 cls._spec: ProcessSpec = cls._spec_class()  # type: ignore
                 cls.__called: bool = False  # type: ignore
                 cls.define(cls._spec)  # type: ignore
-                assert cls.__called, \
-                    'Process.define() was not called by {}\n' \
-                    'Hint: Did you forget to call the superclass method in your define? ' \
-                    'Try: super().define(spec)'.format(cls)
+                assert cls.__called, (
+                    f'Process.define() was not called by {cls}\nHint: Did you forget to call the superclass method in '
+                    'your define? Try: super().define(spec)'
+                )
                 return cls._spec  # type: ignore
             except Exception:
                 del cls._spec  # type: ignore
@@ -296,22 +303,22 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
                 identifier = self._communicator.add_rpc_subscriber(self.message_receive, identifier=str(self.pid))
                 self.add_cleanup(functools.partial(self._communicator.remove_rpc_subscriber, identifier))
             except kiwipy.TimeoutError:
-                self.logger.exception('Process<%s> failed to register as an RPC subscriber', self.pid)
+                self.logger.exception('Process<%s>: failed to register as an RPC subscriber', self.pid)
 
             try:
-                identifier = self._communicator.add_broadcast_subscriber(
-                    self.broadcast_receive, identifier=str(self.pid)
-                )
+                # filter out state change broadcasts
+                subscriber = kiwipy.BroadcastFilter(self.broadcast_receive, subject=re.compile(r'^(?!state_changed).*'))
+                identifier = self._communicator.add_broadcast_subscriber(subscriber, identifier=str(self.pid))
                 self.add_cleanup(functools.partial(self._communicator.remove_broadcast_subscriber, identifier))
             except kiwipy.TimeoutError:
-                self.logger.exception('Process<%s> failed to register as a broadcast subscriber', self.pid)
+                self.logger.exception('Process<%s>: failed to register as a broadcast subscriber', self.pid)
 
         if not self._future.done():
 
             def try_killing(future: futures.Future) -> None:
                 if future.cancelled():
                     if not self.kill('Killed by future being cancelled'):
-                        self.logger.warning('Failed to kill process on future cancel')
+                        self.logger.warning('Process<%s>: Failed to kill process on future cancel', self.pid)
 
             self._future.add_done_callback(try_killing)
 
@@ -481,9 +488,12 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
         return None
 
     def done(self) -> bool:
+        """Return True if the call was successfully killed or finished running.
+
+        .. deprecated:: 0.18.6
+            Use the `has_terminated` method instead
         """
-        Return True if the call was successfully killed or finished running.
-        """
+        warnings.warn('method is deprecated, use `has_terminated` instead', DeprecationWarning)  # pylint: disable=no-member
         return self._state.is_terminal()
 
     # endregion
@@ -525,9 +535,10 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
         try:
             yield None
         finally:
-            assert Process.current() is self, \
-                'Somehow, the process at the top of the stack is not me, ' \
-                'but another process! ({} != {})'.format(self, Process.current())
+            assert Process.current() is self, (
+                'Somehow, the process at the top of the stack is not me, but another process! '
+                f'({self} != {Process.current()})'
+            )
             stack_copy = PROCESS_STACK.get().copy()
             stack_copy.pop()
             PROCESS_STACK.set(stack_copy)
@@ -689,13 +700,16 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
 
         if self._communicator and isinstance(self.state, enum.Enum):
             from_label = cast(enum.Enum, from_state.LABEL).value if from_state is not None else None
-            subject = 'state_changed.{}.{}'.format(from_label, self.state.value)
-            self.logger.info('Broadcasting state change of %d: %s', self.pid, subject)
+            subject = f'state_changed.{from_label}.{self.state.value}'
+            self.logger.info('Process<%s>: Broadcasting state change: %s', self.pid, subject)
             try:
                 self._communicator.broadcast_send(body=None, sender=self.pid, subject=subject)
             except ConnectionClosed:
-                message = 'no connection available to broadcast state change from %s to %s'
-                self.logger.info(message, from_label, self.state.value)
+                message = 'Process<%s>: no connection available to broadcast state change from %s to %s'
+                self.logger.warning(message, self.pid, from_label, self.state.value)
+            except kiwipy.TimeoutError:
+                message = 'Process<%s>: sending broadcast of state change from %s to %s timed out'
+                self.logger.warning(message, self.pid, from_label, self.state.value)
 
     def on_exiting(self) -> None:
         state = self.state
@@ -842,7 +856,7 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
                 try:
                     cleanup()
                 except Exception:  # pylint: disable=broad-except
-                    self.logger.exception('Exception calling cleanup method %s', cleanup)
+                    self.logger.exception('Process<%s>: Exception calling cleanup method %s', self.pid, cleanup)
             self._cleanups = None
         finally:
             self._event_callbacks = {}
@@ -863,7 +877,7 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
         :param msg: the message
         :return: the outcome of processing the message, the return value will be sent back as a response to the sender
         """
-        self.logger.debug("RPC message '%s' received with communicator '%s'", msg, _comm)
+        self.logger.debug("Process<%s>: received RPC message with communicator '%s': %r", self.pid, _comm, msg)
 
         intent = msg[process_comms.INTENT_KEY]
 
@@ -890,7 +904,9 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
         :param msg: the message
         """
         # pylint: disable=unused-argument
-        self.logger.debug("Broadcast message '%s' received with communicator '%s'", body, _comm)
+        self.logger.debug(
+            "Process<%s>: received broadcast message '%s' with communicator '%s': %r", self.pid, subject, _comm, body
+        )
 
         # If we get a message we recognise then action it, otherwise ignore
         if subject == process_comms.Intent.PLAY:
@@ -1032,7 +1048,7 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
 
             return futures.CancellableAction(do_kill, cookie=exception)
 
-        raise ValueError("Got unknown interruption type '{}'".format(type(exception)))
+        raise ValueError(f"Got unknown interruption type '{type(exception)}'")
 
     def _set_interrupt_action(self, new_action: Optional[futures.CancellableAction]) -> None:
         """
@@ -1188,6 +1204,12 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
 
             except KeyboardInterrupt:  # pylint: disable=try-except-raise
                 raise
+            except asyncio.CancelledError:  # pylint: disable=try-except-raise
+                # note this re-raise is only required in python<=3.7,
+                # where asyncio.CancelledError == concurrent.futures.CancelledError
+                # it is encountered when the run_task is cancelled
+                # for python>=3.8 asyncio.CancelledError does not inherit from Exception, so will not be caught below
+                raise
             except Exception:  # pylint: disable=broad-except
                 # Overwrite the next state to go to excepted directly
                 next_state = self.create_state(process_states.ProcessState.EXCEPTED, *sys.exc_info()[1:])
@@ -1254,9 +1276,7 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
             validation_error = port.validate_dynamic_ports({port_name: value})
 
         if validation_error:
-            msg = "Error validating output '{}' for port '{}': {}".format(
-                value, validation_error.port, validation_error.message
-            )
+            msg = f"Error validating output '{value}' for port '{validation_error.port}': {validation_error.message}"
             raise ValueError(msg)
 
         output_namespace = self._outputs
@@ -1302,6 +1322,5 @@ class Process(StateMachine, persistence.Savable, metaclass=ProcessStateMachineMe
             'ctime': self.creation_time,
             'paused': self.paused,
             'process_string': str(self),
-            'state': self.state,
-            'state_info': str(self._state)
+            'state': str(self.state),
         })
